@@ -1,4 +1,4 @@
-from ppycsp3.pycsp3.problems.tests.tester import Tester, run
+from pycsp3.problems.tests.tester import Tester, run
 
 NAME = "pb_testing"
 
