@@ -9,7 +9,8 @@ from pycsp3.classes.auxiliary.ptypes import auto
 from pycsp3.classes.main.variables import Variable
 from pycsp3.tools.inspector import checkType
 from pycsp3.tools.utilities import flatten, is_containing, warning
-from pycsp3.classes.auxiliary.ptypes import TypeCtr, TypeConditionOperator
+from pycsp3.classes.auxiliary.ptypes import TypeCtr, TypeConditionOperator, TypeOrderedOperator
+from pycsp3 import tools
 
 
 class Entity:
@@ -146,6 +147,17 @@ class ECtrs(Entity):
         super().__init__(None)  # no need to have an id here
         assert isinstance(constraints, list)
         self.entities = [c for c in constraints if c is not None]
+        if all(isinstance(c, ECtr) for c in self.entities):
+            t = []
+            for c in self.entities:
+                # if any(c.constraint == cc.constraint for cc in tr):
+                if any(tools.curser.OpOverrider.disable().execute(c.constraint == cc.constraint) for cc in t):
+                    continue
+                t.append(c)
+                if len(t) > 1:
+                    break
+            else:
+                self.entities = t
 
     def _flat_constraints(self, t):
         for e in self.entities:
@@ -389,6 +401,8 @@ class TypeNode(Enum):
     def value_of(v):
         if isinstance(v, TypeNode):
             return v
+        if isinstance(v, TypeOrderedOperator):
+            v = str(v)  # so as to be intercepted just below
         if isinstance(v, str):
             if v in ("<", "lt"):
                 return TypeNode.LT
@@ -457,28 +471,24 @@ class Node(Entity):
             return len(self.sons) == len(other.sons) and all(self.sons[i].eq__safe(other.sons[i]) for i in range(len(self.sons)))
         return self.sons.eq__safe(other.sons) if isinstance(self.sons, Variable) else self.sons == other.sons
 
-    def __strsmart__(self):
+    def __str_hybrid__(self):
         if self.type.is_leaf():
             if self.type == TypeNode.COL:
-                return "%" + str(self.sons)
+                return "c" + str(self.sons)  # return "%" + str(self.sons)
             return str(self.sons)
         if self.type == TypeNode.ADD or self.type == TypeNode.SUB:
-            msg = "Smart tuple must be of the form {eq|lt|le|ge|gt|ne}{var|integer}{+|-}{var|integer}"
+            msg = "An hybrid tuple must be of the form {eq|lt|le|ge|gt|ne}{var|integer}{+|-}{var|integer}"
             assert len(self.sons) == 2, msg
             assert self.sons[0].type in (TypeNode.INT, TypeNode.COL) and self.sons[1].type in (TypeNode.INT, TypeNode.COL), msg
-            return self.sons[0].__strsmart__() + ("+" if self.type == TypeNode.ADD else "-") + self.sons[1].__strsmart__()
+            return self.sons[0].__str_hybrid__() + ("+" if self.type == TypeNode.ADD else "-") + self.sons[1].__str_hybrid__()
         else:
-            assert False, "Smart tuple must be of the form col(x)[+or-][integer]"
+            assert False, "An hybrid tuple must be of the form col(x)[+or-][integer]"
 
     def __str__(self):
+        if self.type == TypeNode.COL:
+            return "%" + str(self.sons)
         return str(self.sons) if self.type.is_leaf() else str(self.type) + "(" + ",".join(str(son) for son in self.sons) + ")"
 
-    def _product(t):
-        p = 1
-        for i in t:
-            p *= i
-        return p
-
     def possible_values(self):
         if self.type.is_predicate_operator():
             return range(0, 2)  # we use a range instead of [0,1] because it simplifies computation (see code below)
@@ -506,9 +516,9 @@ class Node(Entity):
             if self.type == TypeNode.SUB:
                 return add_range(pv1, neg_range(pv2)) if all_ranges else possible_range({v1 - v2 for v1 in pv1 for v2 in pv2})
             if self.type == TypeNode.DIV:
-                return possible_range({v1 // v2 for v1 in pv1 for v2 in pv2})
+                return possible_range({v1 // v2 for v1 in pv1 for v2 in pv2 if v2 != 0})
             if self.type == TypeNode.MOD:
-                return possible_range({v1 % v2 for v1 in pv1 for v2 in pv2})
+                return possible_range({v1 % v2 for v1 in pv1 for v2 in pv2 if v2 != 0})
             if self.type == TypeNode.POW:
                 return possible_range({v1 ** v2 for v1 in pv1 for v2 in pv2}, control_int=True)
             if self.type == TypeNode.DIST:
@@ -525,7 +535,15 @@ class Node(Entity):
             if self.type == TypeNode.ADD:
                 return reduce(add_range, pvs) if all_ranges else possible_range({sum(p) for p in product(*(pv for pv in pvs))})
             if self.type == TypeNode.MUL:
-                return possible_range({self._product(p) for p in product(*(pv for pv in pvs))})  # or numpy.prod ?
+                def multiply(l):
+                    res = 1
+                    for v in l:
+                        res *= v
+                    return res
+
+                if all_ranges and all(pv.start >= 0 and pv.step == 1 for pv in pvs):
+                    return range(multiply(pv.start for pv in pvs), multiply(pv.stop - 1 for pv in pvs) + 1)
+                return possible_range({multiply(p) for p in product(*(pv for pv in pvs))})  # or numpy.prod ?
             # TODO: in case of all_ranges being False, possibility of improving the efficiency of the code below for MIN and MAX
             if self.type == TypeNode.MIN:
                 return range(min(pv.start for pv in pvs), min(pv.stop for pv in pvs)) if all_ranges \
@@ -662,6 +680,10 @@ class Node(Entity):
         args = flatten(Node.build(TypeNode.SET, arg) if isinstance(arg, (set, range, frozenset)) else arg for arg in args)
         assert type.is_valid_arity(len(args)), "Problem: Bad arity for node " + type.name + ". It is " + str(
             len(args)) + " but it should be between " + str(type.min_arity) + " and " + str(type.max_arity)
+        # Do we activate these simple modifications below?
+        # if len(args) == 2 and isinstance(args[0], Variable) and isinstance(args[1], int):
+        #     if (args[1] == 1 and type in (TypeNode.MUL, TypeNode.DIV)) or (args[1] == 0 and type in (TypeNode.ADD, TypeNode.SUB)):
+        #         return Node(TypeNode.VAR,args[0])
         node = Node(type, Node._create_sons(*args))
         if type == TypeNode.EQ and all(son.type.is_predicate_operator() for son in node.sons):
             node = Node(TypeNode.IFF, node.sons)
@@ -694,3 +716,13 @@ class Node(Entity):
     @staticmethod
     def disjunction(*args):
         return Node._and_or(TypeNode.OR, *args)
+
+    @staticmethod
+    def in_range(x, r):
+        assert isinstance(r, range) and r.step == 1
+        return Node.build(TypeNode.AND, Node.build(TypeNode.GE, x, r.start), Node.build(TypeNode.LT, x, r.stop))
+
+    @staticmethod
+    def not_in_range(x, r):
+        assert isinstance(r, range) and r.step == 1
+        return Node.build(TypeNode.OR, Node.build(TypeNode.LT, x, r.start), Node.build(TypeNode.GE, x, r.stop))
