@@ -16,7 +16,7 @@ from pycsp3.tools.aggregator import build_similar_constraints
 from pycsp3.tools.compactor import build_compact_forms
 from pycsp3.tools.curser import OpOverrider, convert_to_namedtuples, is_namedtuple
 from pycsp3.tools.slider import handle_slides
-from pycsp3.tools.utilities import Stopwatch, GREEN, WHITE, Error
+from pycsp3.tools.utilities import Stopwatch, GREEN, WHITE, Error, error
 from pycsp3.tools.xcsp import build_document
 
 None_Values = ['None', '', 'null']  # adding 'none'?
@@ -31,45 +31,66 @@ class Compilation:
     stopwatch = None
     stopwatch2 = None
     done = False
-    user_filename = None
+    pathname = ""
+    filename = ""
 
     @staticmethod
     def load(console=False):
         _load(console=console)
 
     @staticmethod
-    def set_filename(_user_filename):
-        if _user_filename is None:
-            Compilation.user_filename = None
+    def set_path_file_name(name):
+        if name is None or len(name) == 0:
+            Compilation.pathname = ""
+            Compilation.filename = ""
         else:
-            Compilation.user_filename = _user_filename if _user_filename.endswith(".xml") else _user_filename + ".xml"
+            if os.path.isdir(name) and not name.endswith(os.sep):
+                name += os.sep
+            i = name.rfind(os.sep)
+            if i != -1:
+                Compilation.pathname = name[:i + len(os.sep)]
+                name = name[i + len(os.sep):]
+            else:
+                Compilation.pathname = ""
+            Compilation.filename = name if len(name) == 0 or name.endswith(".xml") else name + ".xml"
 
     @staticmethod
-    def compile(filename=None, disabling_opoverrider=False, verbose=1):
-        Compilation.set_filename(filename)
+    def compile(filename="", disabling_opoverrider=False, verbose=1):
+        Compilation.set_path_file_name(filename)
         return _compile(disabling_opoverrider, verbose=verbose)
 
 
 def _load_options():
-    options.set_values("data", "dataparser", "dataexport", "dataformat", "variant", "checker", "solver", "output")
-    options.set_flags("dataexport", "solve", "display", "verbose", "lzma", "sober", "ev", "safe", "recognizeSlides", "keepSmartConditions",
-                      "restrictTablesWrtDomains", "dontcompactValues", "usemeta", "debug")
+    options.set_values("data", "dataparser", "dataexport", "dataformat", "variant", "tocsp", "checker", "solver", "output")
+    options.set_flags("dataexport", "solve", "display", "verbose", "lzma", "sober", "ev", "safe", "recognizeSlides", "keepHybrid",
+                      "keepSmartTransitions", "restrictTablesWrtDomains", "dontruncompactor", "dontcompactValues", "groupsumcoeffs",
+                      "usemeta", "debug", "mini", "uncurse")
     if options.checker is None:
         options.checker = "fast"
     assert options.checker in {"complete", "fast", "none"}
     options.parse(sys.argv[1:])
 
 
+def _basic_token(name):
+    pos = name.rfind(os.sep)
+    if pos != -1:
+        name = name[pos + 1:]
+    pos = name.rfind(".")
+    if pos != -1:
+        name = name[:pos]
+    return name
+
+
 def _load_model():
     try:
         name = sys.argv[0]
         assert name.strip().endswith(".py"), "The first argument has to be a python file." + str(name)
-        model_string = name[name.rfind(os.sep) + 1:name.rfind(".")]
+        model_string = _basic_token(name)
         specification = util.spec_from_file_location("", name)
         model = util.module_from_spec(specification)
         return model, model_string
     except Exception:
-        usage("It was not possible to read the file: " + sys.argv[0])
+        error("It was not possible to read the file: " + sys.argv[0])
         raise
 
 
@@ -95,7 +116,7 @@ def _load_data():
                 assert os.path.exists(arg), "The file " + arg + " does not exist (in the specified directory)." + str(os.path)
                 with open(arg) as f:
                     compilation_data.update(json.loads(f.read(), object_pairs_hook=OrderedDict))
-                    s += "-" + arg.split(os.sep)[-1:][0].split(".")[:1][0]
+                    s += "-" + _basic_token(arg)
         return compilation_data, s
 
     data = options.data
@@ -105,10 +126,10 @@ def _load_data():
         if data.startswith("http"):
             from urllib.request import urlopen
             # example: python Nonogram.py -data=https://www.cril.univ-artois.fr/~lecoutre/heart.json
-            return json.loads(urlopen(data).read(), object_pairs_hook=OrderedDict), "-" + data.split(os.sep)[-1:][0].split(".")[:1][0]
+            return json.loads(urlopen(data).read(), object_pairs_hook=OrderedDict), "-" + _basic_token(data)
         assert os.path.exists(data), "The file " + data + " does not exist (in the specified directory)."
         with open(data) as f:
-            return json.loads(f.read(), object_pairs_hook=OrderedDict), "-" + data.split(os.sep)[-1:][0].split(".")[:1][0]
+            return json.loads(f.read(), object_pairs_hook=OrderedDict), "-" + _basic_token(data)
     compilation_data = OrderedDict()  # the object used for recording the data, available in the model
     # if '{' in data and '}' in data:
     #    compilation_data = json.loads(data, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()), object_pairs_hook=OrderedDict)
@@ -146,12 +167,18 @@ def _load_dataparser(parser_file, data_value):
         compilation_data = parsing.register_fields(data_value)  # the object used for recording data is returned, available in the model
         specification = util.spec_from_file_location("", parser_file)
         specification.loader.exec_module(util.module_from_spec(specification))
-        string_data = "-" + options.data.split(os.sep)[-1:][0].split(".")[:1][0] if options.data else None
+        string_data = None
+        if options.data:
+            if options.data[0] == '[':
+                assert options.data[-1] == ']'
+                string_data = "-" + "-".join(_basic_token(tok) for tok in options.data[1:-1].split(","))
+            else:
+                string_data = "-" + _basic_token(options.data) if options.data else None
         if string_data is None:
             string_data = Compilation.string_data if Compilation.string_data else ""  # in case data are recorded through the dataparser (after asking the user)
         return compilation_data, string_data
     except Exception:
-        usage("It was not possible to correctly read the file: " + parser_file)
+        error("It was not possible to correctly read the file: " + parser_file)
         raise
 
 
@@ -167,6 +194,7 @@ def _load(*, console=False):
             Compilation.data, Compilation.string_data = _load_data()
         Compilation.data = convert_to_namedtuples(Compilation.data)
         Compilation.string_data = Compilation.string_data.replace("/", "-")
+        Compilation.original_data = Compilation.data
         if len(Compilation.data) == 0:
             Compilation.data = None
         elif len(Compilation.data) == 1:
@@ -206,7 +234,7 @@ def load_json_data(filename, *, storing=False):
         data = data[0]  # the value instead of a tuple of size 1
     if storing:
         Compilation.data = data
-        Compilation.string_data = "-" + filename.split(os.sep)[-1:][0].split(".")[:1][0]
+        Compilation.string_data = "-" + _basic_token(filename)
     return data
 
 
@@ -240,29 +268,27 @@ def _compile(disabling_opoverrider=False, verbose=1):
     if disabling_opoverrider:
         OpOverrider.disable()
 
-    if Compilation.user_filename is None and options.output is not None:
-        Compilation.set_filename(options.output)
-    if Compilation.user_filename is not None:
-        if options.output is None and options.verbose:
-            print("  * User-defined XML file name:", Compilation.user_filename)
-        filename = Compilation.user_filename
-        if filename.endswith(".xml"):
-            filename_prefix = filename[:-4]  # can be useful if data are exported
+    if Compilation.filename is "" and options.output is not None:  # why the first part of the condition?
+        Compilation.set_path_file_name(options.output)
+    if len(Compilation.filename) > 0:
+        if Compilation.filename.endswith(".xml"):
+            filename_prefix = Compilation.filename[:-4]  # can be useful if data are exported
+        fullname = Compilation.pathname + Compilation.filename
     else:
         same_prefix = Compilation.string_data.startswith("-" + Compilation.string_model)
         suffix = Compilation.string_data if not same_prefix else Compilation.string_data[1 + len(Compilation.string_model):]
         filename_prefix = Compilation.string_model + ("-" + options.variant if options.variant else "") + suffix
-        filename = filename_prefix + ".xml"
+        fullname = Compilation.pathname + filename_prefix + ".xml"
 
     stopwatch = Stopwatch()
-    if options.verbose:
-        print("  PyCSP3 (Python:" + platform.python_version() + ", Path:" + os.path.abspath(__file__) + ")\n")
+    options.verbose and print("  PyCSP3 (Python:" + platform.python_version() + ", Path:" + os.path.abspath(__file__) + ")\n")
     build_similar_constraints()
     options.verbose and print("\tWCK for generating groups:", stopwatch.elapsed_time(reset=True), "seconds")
     handle_slides()
     options.verbose and print("\tWCK for handling slides:", stopwatch.elapsed_time(reset=True), "seconds")
-    build_compact_forms()
-    options.verbose and print("\tWCK for compacting forms:", stopwatch.elapsed_time(reset=True), "seconds")
+    if options.dontruncompactor is False:
+        build_compact_forms()
+        options.verbose and print("\tWCK for compacting forms:", stopwatch.elapsed_time(reset=True), "seconds")
 
     root = build_document()
     if root is not None:
@@ -270,29 +296,39 @@ def _compile(disabling_opoverrider=False, verbose=1):
         if options.display:
             print("\n", pretty_text)
         else:
-            with open(filename, "w") as f:
+            with open(fullname, "w") as f:
                 f.write(pretty_text)
                 if verbose > 0:
-                    print("  * Generating the file " + filename + " completed in " + GREEN + Compilation.stopwatch.elapsed_time() + WHITE + " seconds.")
+                    print("  * Generating the file " + fullname + " completed in " + GREEN + Compilation.stopwatch.elapsed_time() + WHITE + " seconds.")
         if options.lzma:
-            with lzma.open(filename + ".lzma", "w") as f:
+            with lzma.open(fullname + ".lzma", "w") as f:
                 f.write(bytes(pretty_text, 'utf-8'))
-                print("\tGeneration of the file " + filename + ".lzma completed.\n")
+                print("\tGeneration of the file " + fullname + ".lzma completed.\n")
         options.verbose and print("\tWCK for generating files:", stopwatch.elapsed_time(reset=True), "seconds")
 
     if options.dataexport:
         if isinstance(options.dataexport, bool):
-            json_prefix = options.data.split("/")[-1:][0].split(".")[:1][0] if options.dataparser else filename_prefix
+            if options.data is None:
+                json_prefix = "data" + Compilation.string_data
+            elif options.dataparser is None:
+                json_prefix = filename_prefix
+            else:
+                if options.data[0] == '[':
+                    assert options.data[-1] == ']'
+                    json_prefix = "-".join(_basic_token(tok) for tok in options.data[1:-1].split(","))
+                else:
+                    json_prefix = _basic_token(options.data) if options.data else None
+                # json_prefix = options.data.split(os.sep)[-1].split(".")[:1][0] if options.dataparser else filename_prefix
             # TODO if data are given with name as e.g., in [k=3,l=9,b=0,r=0,v=9] for Bibd, maybe we should sort them
         else:
             json_prefix = str(options.dataexport)
-        with open(json_prefix + '.json', 'w') as f:
-            json.dump(prepare_for_json(Compilation.data), f)
-        print("  Saving data in the file " + json_prefix + '.json' + " completed.")
+        with open(Compilation.pathname + json_prefix + '.json', 'w') as f:
+            json.dump(prepare_for_json(Compilation.original_data if Compilation.original_data else Compilation.data), f)
+        print("  Saving data in the file " + (Compilation.pathname + json_prefix) + '.json' + " completed.")
 
     Compilation.done = True
     cop = root is not None and root.attrib and root.attrib["type"] == "COP"
-    return filename, cop
+    return fullname, cop
 
 
 def usage(message):
