@@ -47,7 +47,8 @@ minimize(
 )
 
 """ Comments
-1) the call to Slide can be avoided, but the interest is two-fold:
+1) the call to Slide can be avoided (it was the case in the 2022 competition), but the interest is two-fold:
   - getting a more compact XCSP3 file
   - letting solvers the possibility of exploiting the meta-constraint Slide (stronger filtering)
 """
+
