@@ -5,7 +5,14 @@ from pycsp3.dashboard import options
 from pycsp3.tools.utilities import flatten, is_windows
 
 if not is_windows():
-    import readline
+    try:
+        import readline
+    except ImportError:
+        if "Intel" in sys.version:
+            s = 'With Intel compilers, the module readline must be installed by the user.'
+        else:
+            s = 'Problem when importing the module readline'
+        raise ImportError(s)
 
 fileToTrace = [arg for arg in sys.argv if arg.endswith(".py") and "compiler.py" not in arg] + ["<stdin>"]
 
@@ -179,11 +186,11 @@ def _delete_bracket_part(code, nb_parameters):
     # else:
     code = [line.strip() if is_comment_line(line) else _remove_matching_brackets(line).strip() for line in code]
     if len(code) > 0 and len(code[-1]) > 0 and code[-1][-1] == ',':  # removing useless trailing comma of specify() if any
-        code[-1] = code[:-1]
+        code[-1] = code[-1][:-1]
     return code
 
 
-def comments_and_tags_of_parameters_of(*, function_name, args):
+def comments_and_tags_of_parameters_of(*, function_name, args, no_extraction=False):
     if len(args) == 0:
         return [], [], [], []
 
@@ -191,8 +198,10 @@ def comments_and_tags_of_parameters_of(*, function_name, args):
     comments2 = [[""] for _ in args]  # comments at second level (i.e, for elements of intern lists)
     tags1 = [""] * len(args)  # tags at first level
     tags2 = [[""] for _ in args]  # tags at second level
-    code = list(reversed(_extract_code(function_name)))
+    if no_extraction:
+        return comments1, comments2, tags1, tags2
 
+    code = list(reversed(_extract_code(function_name)))
     are_empty_lines = [is_empty_line(line) for line in code]
 
     code = _delete_bracket_part(code, len(args))
@@ -269,7 +278,7 @@ def extract_declaration_for(function_name):
             if "=" in line[:pos]:
                 break
     else:
-        return None #assert False, " the object returned by " + function_name + " should be assigned to a variable"
+        return None  # assert False, " the object returned by " + function_name + " should be assigned to a variable"
     declaration = line[:pos].strip()
     if declaration.endswith("pycsp3."):  # if the name of the package is present as a suffix, we remove it
         declaration = declaration[:-8]
@@ -285,7 +294,7 @@ def extract_declaration_for(function_name):
         return declaration
     elif function_name == "VarArray":
         assert ")" not in declaration
-        return declaration if "," not in declaration else declaration.split(",")
+        return declaration if "," not in declaration else [v.strip() for v in declaration.split(",")]
 
 
 # def docstringOf(name):
