@@ -8,8 +8,10 @@ from multiprocessing import cpu_count, Pool
 from time import time
 
 from pycsp3.classes.auxiliary import conditions
+from pycsp3.classes.auxiliary.ptypes import TypeSquareSymmetry, TypeRectangleSymmetry
 from pycsp3.classes.main.domains import Domain
 from pycsp3.dashboard import options
+from pycsp3 import tools
 
 
 class Stopwatch:
@@ -75,15 +77,17 @@ def flatten(*args, keep_none=False):
                 t.append(arg)
         elif isinstance(arg, (str, range, Domain)):  # Iterable but must be appended, not extended
             t.append(arg)
+        elif isinstance(arg, types.GeneratorType):
+            res = list(arg)
+            if len(res) > 0:
+                t.extend(flatten(*res, keep_none=keep_none))
         elif isinstance(arg, Iterable):
             t.extend(flatten(*arg, keep_none=keep_none))
-        elif isinstance(arg, types.GeneratorType):
-            t.extend(flatten(*list(arg), keep_none=keep_none))
         else:
             t.append(arg)
     # if len(args) == 1:
     #     flatten.cache[id(args[0])] = t
-    return t
+    return tools.curser.cp_array(t)  # previously: return t
 
 
 def is_containing(l, types, *, check_first_only=False):
@@ -117,16 +121,16 @@ def unique_type_in(l, tpe=None):
         return None if l is None else type(l) if tpe is None else tpe if isinstance(l, tpe) else False
 
 
-def is_1d_list(l, types=None):
-    if types is not None and len(l) == 0:
-        return False
-    return isinstance(l, list) and all(isinstance(v, types) if types else not isinstance(v, list) for v in l)
-
-
 def is_1d_tuple(l, types):
-    if types is not None and len(l) == 0:
+    if not isinstance(l, tuple) or types is not None and len(l) == 0:
         return False
-    return isinstance(l, tuple) and all(isinstance(v, types) for v in l)
+    return all(isinstance(v, types) for v in l)
+
+
+def is_1d_list(l, types=None):
+    if not isinstance(l, list) or types is not None and len(l) == 0:
+        return False
+    return all(isinstance(v, types) if types else not isinstance(v, list) for v in l)
 
 
 def is_2d_list(m, types=None):
@@ -141,9 +145,17 @@ def is_square_matrix(m, types=None):
     return is_matrix(m, types) and len(m) == len(m[0])
 
 
+def is_3d_list(c, types=None):
+    return isinstance(c, list) and all(is_2d_list(m, types) for m in c)
+
+
+def is_cube(c, types=None):
+    return is_3d_list(c, types) and all(len(m) == len(c[0]) for m in c) and all(all(len(l) == len(m[0]) for l in m) for m in c)
+
+
 def alphabet_positions(s):
     '''
-    Returns a list with the indexes of the letters (with respect to the 26 letters of the Latin alphabet) of the specified string.
+    Returns a tuple with the indexes of the letters (with respect to the 26 letters of the Latin alphabet) of the specified string.
 
     @param s: a string
     '''
@@ -193,6 +205,21 @@ def integer_scaling(values):
     return [int(w * (10 ** scale)) for w in [Decimal(v) for v in values]]
 
 
+def decrement(t):
+    if isinstance(t, types.GeneratorType):
+        t = list(t)
+    assert isinstance(t, list)
+    for i in range(len(t)):
+        if isinstance(t[i], list):
+            t[i] = decrement(t[i])
+        elif isinstance(t[i], tuple):
+            t[i] = tuple(v - 1 for v in t[i])
+        else:
+            assert isinstance(t[i], int)
+            t[i] -= 1
+    return t
+
+
 def matrix_to_string(m):
     return "".join(["(" + ",".join([str(v) for v in t]) + ")" for t in m])
     # return "\n" + "\n".join(["\t(" + ",".join([str(v) for v in t]) + ")" for t in m]) + "\n"
@@ -201,10 +228,10 @@ def matrix_to_string(m):
 def table_to_string(table, restricting_domains=None, *, parallel=False):
     def _tuple_to_string(t):
         return "(" + ",".join(
-            str(v) if isinstance(v, (int)) else
-            str(set(v)) if isinstance(v, tuple) else
+            str(v) if isinstance(v, int) else
+            ("{" + ",".join(str(w) for w in sorted(v)) + "}") if isinstance(v, (tuple,list,set,frozenset)) else
             conditions.inside(v).str_tuple() if isinstance(v, range) else
-            v if isinstance(v, (str)) else
+            v if isinstance(v, str) else
             "*" if v == ANY else v.str_tuple()
             for v in t) + ")"
 
@@ -261,6 +288,29 @@ def integers_to_string(numbers):
     return ' '.join(str(i[0]) if len(i) == 1 else str(i[0]) + ('..' if i[0] != i[1] - 1 else ' ') + str(i[1]) for i in t)
 
 
+def _remove_condition_nodes_of_table(table, doms):  # convert hybrid binary and ternary conditions into ordinary tuples
+    def _remove_condition_nodes_of_tuple():
+        r = len(t)
+        pos = next((i for i in range(r) if isinstance(t[i], conditions.ConditionNode)), -1)
+        if pos == -1:
+            return t
+        ind, res = t[pos].evaluate(t, doms)
+        assert len(ind) in (1, 2)
+        return (tuple(v if i == pos else st[0] if i == ind[0] else st[1] if len(ind) == 2 and i == ind[1] else t[i] for i in range(r))
+                for v in doms[pos] for st in res if t[pos].operator.check(v, st[-1]))
+
+    done = []
+    todo = table
+    while len(todo) > 0:
+        t = todo.pop(0)
+        res = _remove_condition_nodes_of_tuple()
+        if res is t:
+            done.append(t)
+        else:
+            todo.extend(res)
+    return done
+
+
 def to_ordinary_table(table, domains, *, starred=False):
     """
     Converts the specified table that may contain hybrid restrictions and stars into an ordinary table (or a starred table).
@@ -274,23 +324,54 @@ def to_ordinary_table(table, domains, *, starred=False):
     """
     doms = [range(d) if isinstance(d, int) else d.all_values() if isinstance(d, Domain) else d for d in domains]
     tbl = set()
+    contains_node_condition = False
     if starred:
         for t in table:
             if any(isinstance(v, conditions.Condition) for v in t):  # v may be a Condition object (with method 'filtering')
-                tbl.update(product(
-                    *({v} if isinstance(v, int) or v == ANY else [w for w in v if w in doms[i]] if isinstance(v, (list, tuple))
-                    else v.filtering(doms[i]) for i, v in enumerate(t))))
+                if contains_node_condition is False and any(isinstance(v, conditions.ConditionNode) for v in t):
+                    contains_node_condition = True
+                l = ({v} if isinstance(v, int) or v == ANY else [w for w in v if w in doms[i]] if isinstance(v, (list, tuple, set, frozenset)) else v.filtering(
+                    doms[i]) for i, v in enumerate(t))
+                tbl.update(product(*l))
             else:
                 tbl.add(t)
     else:
         for t in table:
             if any(v == ANY or isinstance(v, conditions.Condition) for v in t):  # v may be a ConditionValue object (with method 'filtering')
+                if contains_node_condition is False and any(isinstance(v, conditions.ConditionNode) for v in t):
+                    contains_node_condition = True
                 tbl.update(product(*(
-                    {v} if isinstance(v, int) else doms[i] if v == ANY else [w for w in v if w in doms[i]] if isinstance(v, (list, tuple))
+                    {v} if isinstance(v, int) else doms[i] if v == ANY else [w for w in v if w in doms[i]] if isinstance(v, (list, tuple, set, frozenset))
                     else v.filtering(doms[i]) for i, v in enumerate(t))))
             else:
                 tbl.add(t)
-    return tbl
+    return tbl if not contains_node_condition else _remove_condition_nodes_of_table(list(tbl),
+                                                                                    doms)  # this must be performed after removing other kind of conditions
+
+
+def _non_overlapping_tuples_for(t, dom1, dom2, offset, first, x_axis=None):
+    for va in dom1:
+        for vb in reversed(dom2.all_values()):
+            if va + offset > vb:
+                break
+            sub = (va, vb) if first else (vb, va)
+            t.append(sub if x_axis is None else sub + (ANY, ANY) if x_axis else (ANY, ANY) + sub)
+
+
+def to_starred_table_for_no_overlap1(x1, x2, w1, w2):
+    t = []
+    _non_overlapping_tuples_for(t, x1.dom, x2.dom, w1, True)
+    _non_overlapping_tuples_for(t, x2.dom, x1.dom, w2, False)
+    return t
+
+
+def to_starred_table_for_no_overlap2(x1, x2, y1, y2, w1, w2, h1, h2):
+    t = []
+    _non_overlapping_tuples_for(t, x1.dom, x2.dom, w1, True, True)
+    _non_overlapping_tuples_for(t, x2.dom, x1.dom, w2, False, True)
+    _non_overlapping_tuples_for(t, y1.dom, y2.dom, h1, True, False)
+    _non_overlapping_tuples_for(t, y2.dom, y1.dom, h2, False, False)
+    return t
 
 
 def display_constraints(ctr_entities, separator=""):
@@ -304,7 +385,9 @@ def display_constraints(ctr_entities, separator=""):
 
 
 def structured_list(m, level=1):
-    if m is None or not isinstance(m, (list, tuple)) or len(m) == 0:
+    if m is None or len(m) == 0:
+        return "[]"
+    if not isinstance(m, (list, tuple)):
         return str(m)
     gap = "  "
     if isinstance(m[0], (list, tuple)):
@@ -350,3 +433,76 @@ def error(s):
 def error_if(test, s):
     if test:
         error(s)
+
+
+polyominoes = [
+    {},  # dummy entry
+    {  # Monomino
+        "p1": [(0, 0)]
+    },
+    {  # Domino
+        "p2": [(0, 0), (1, 0)]
+    },
+    {  # Trominoes
+        "Bar": [(0, 0), (1, 0), (2, 0)],
+        "Right": [(0, 0), (1, 0), (1, 1)]
+    },
+    {  # Tetrominoes
+        "Square": [(0, 0), (0, 1), (1, 0), (1, 1)],
+        "T": [(0, 0), (0, 1), (0, 2), (1, 1)],
+        "Skew": [(0, 0), (0, 1), (1, 1), (1, 2)],
+        "L": [(0, 0), (0, 1), (0, 2), (-1, 2)],
+        "Bar": [(0, 0), (0, 1), (0, 2), (0, 3)]
+    },
+    {  # Pentominoes
+        "F": [(0, 0), (0, 1), (1, -1), (1, 0), (2, 0)],
+        "I": [(0, 0), (1, 0), (2, 0), (3, 0), (4, 0)],
+        "L": [(0, 0), (1, 0), (2, 0), (3, 0), (3, 1)],
+        "N": [(0, 0), (1, -1), (1, 0), (2, -1), (3, -1)],
+        "P": [(0, 0), (1, -1), (1, 0), (2, -1), (2, 0)],
+        "T": [(0, 0), (0, 1), (0, 2), (1, 1), (2, 1)],
+        "U": [(0, 0), (1, 0), (1, 1), (1, 2), (0, 2)],
+        "V": [(0, 0), (1, 0), (2, 0), (2, 1), (2, 2)],
+        "W": [(0, 0), (1, 0), (1, 1), (2, 1), (2, 2)],
+        "X": [(0, 0), (1, -1), (1, 0), (1, 1), (2, 0)],
+        "Y": [(0, 0), (1, -1), (1, 0), (2, 0), (3, 0)],
+        "Z": [(0, 0), (0, 1), (1, 1), (2, 1), (2, 2)]
+    },
+    {  # Hexominoes
+        "A": [(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (2, 0)],
+        "C": [(0, 0), (0, 1), (1, 0), (2, 0), (3, 0), (3, 1)],
+        "D": [(0, 0), (0, 1), (0, 2), (0, 3), (1, 1), (1, 2)],
+        "E": [(0, 0), (0, 2), (1, 0), (1, 1), (1, 2), (2, 1)],
+        "highF": [(0, 0), (0, 1), (0, 2), (0, 3), (-1, 3), (1, 2)],
+        "LowF": [(0, 0), (1, 0), (1, 1), (1, 2), (1, 3), (2, 2)],
+        "G": [(0, 0), (0, 2), (0, 3), (1, 0), (1, 1), (1, 2)],
+        "H": [(0, 0), (0, 1), (0, 2), (1, 1), (2, 0), (2, 1)],
+        "I": [(0, 0), (0, 1), (0, 2), (0, 3), (0, 4), (0, 5)],
+        "J": [(0, 0), (0, 1), (0, 2), (1, 0), (2, 0), (2, 1)],
+        "K": [(0, 0), (0, 1), (1, 0), (1, 1), (1, 2), (2, 1)],
+        "L": [(0, 0), (0, 1), (1, 0), (2, 0), (3, 0), (4, 0)],
+        "M": [(0, 0), (0, 1), (0, 2), (-1, 2), (-1, 3), (1, 0)],
+        "longN": [(0, 0), (0, 1), (0, 2), (0, 3), (-1, 3), (-1, 4)],
+        "shortN": [(0, 0), (0, 1), (0, 2), (-1, 1), (-1, 2), (-1, 3)],
+        "O": [(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (1, 2)],
+        "P": [(0, 0), (0, 1), (0, 2), (0, 3), (1, 2), (1, 3)],
+        "Q": [(0, 0), (1, 0), (1, 1), (1, 2), (2, 1), (2, 2)],
+        "R": [(0, 0), (1, 0), (1, 1), (1, 2), (2, 0), (2, 1)],
+        "S": [(0, 0), (0, 1), (0, 2), (-1, 2), (-1, 3), (-1, 4)],
+        "tallT": [(0, 0), (0, 1), (0, 2), (1, 1), (2, 1), (3, 1)],
+        "shortT": [(0, 0), (0, 1), (0, 2), (0, 3), (1, 2), (2, 2)],
+        "U": [(0, 0), (0, 1), (0, 2), (0, 3), (1, 0), (1, 2)],
+        "V": [(0, 0), (0, 1), (0, 2), (1, 0), (2, 0), (3, 0)],
+        "Wa": [(0, 0), (0, 1), (0, 2), (-1, 2), (-1, 3), (-2, 3)],
+        "Wb": [(0, 0), (0, 1), (-1, 1), (-1, 2), (-2, 2), (-2, 3)],
+        "Wc": [(0, 0), (0, 1), (1, 1), (1, 2), (1, 3), (2, 2)],
+        "X": [(0, 0), (0, 1), (0, 2), (0, 3), (-1, 2), (1, 2)],
+        "italicX": [(0, 0), (0, 1), (0, 2), (0, 3), (-1, 1), (1, 2)],
+        "highY": [(0, 0), (0, 1), (0, 2), (0, 3), (0, 4), (1, 3)],
+        "lowY": [(0, 0), (0, 1), (0, 2), (0, 3), (0, 4), (1, 2)],
+        "tallZ": [(0, 0), (0, 1), (1, 0), (2, 0), (3, -1), (3, 0)],
+        "shortZ": [(0, 0), (0, 1), (1, 0), (2, -2), (2, -1), (2, 0)],
+        "high4": [(0, 0), (1, 0), (1, 1), (1, 2), (2, 1), (3, 1)],
+        "low4": [(0, 0), (1, 0), (2, 0), (2, 1), (2, 2), (3, 1)]
+    }
+]
